"""
@Author: obstacles
@Time:  2025-05-09 14:11
@Description:  
"""

from puti.constant.base import Base


class Utilities(Base):
    CELERY = ('celery', 'celery module')
    MYSQL = ('mysql', 'mysql module')
